@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* calendar-component.c
  *
- * Copyright (C) 2003  Ettore Perazzoli
+ * Copyright (C) 2003  Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -30,14 +30,18 @@
 #include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-i18n.h>
 #include <bonobo/bonobo-exception.h>
+#include "e-pub-utils.h"
+#include "calendar-config-keys.h"
 #include "calendar-config.h"
 #include "calendar-component.h"
 #include "calendar-commands.h"
+#include "control-factory.h"
 #include "gnome-cal.h"
 #include "migration.h"
 #include "e-comp-editor-registry.h"
 #include "comp-util.h"
-#include "dialogs/new-calendar.h"
+#include "common/authentication.h"
+#include "dialogs/calendar-setup.h"
 #include "dialogs/comp-editor.h"
 #include "dialogs/copy-source-dialog.h"
 #include "dialogs/event-editor.h"
@@ -45,9 +49,10 @@
 
 
 /* IDs for user creatable items */
-#define CREATE_EVENT_ID "event"
-#define CREATE_MEETING_ID "meeting"
+#define CREATE_EVENT_ID        "event"
+#define CREATE_MEETING_ID      "meeting"
 #define CREATE_ALLDAY_EVENT_ID "allday-event"
+#define CREATE_CALENDAR_ID      "calendar"
 
 #define PARENT_TYPE bonobo_object_get_type ()
 static BonoboObjectClass *parent_class = NULL;
@@ -56,12 +61,15 @@ struct _CalendarComponentPrivate {
 	char *config_directory;
 
 	GConfClient *gconf_client;
+	int gconf_notify_id;
 	ESourceList *source_list;
 	GSList *source_selection;
 	
 	GnomeCalendar *calendar;
 	GtkWidget *source_selector;
 
+	BonoboControl *view_control;
+
 	ECal *create_ecal;
 
 	GList *notifications;
@@ -74,24 +82,6 @@ extern ECompEditorRegistry *comp_editor_registry;
 
 /* Utility functions.  */
 
-static void
-add_uri_for_source (ESource *source, GnomeCalendar *calendar)
-{
-	char *uri = e_source_get_uri (source);
-
-	gnome_calendar_add_event_uri (calendar, uri);
-	g_free (uri);
-}
-
-static void
-remove_uri_for_source (ESource *source, GnomeCalendar *calendar)
-{
-	char *uri = e_source_get_uri (source);
-
-	gnome_calendar_remove_event_uri (calendar, uri);
-	g_free (uri);
-}
-
 static gboolean
 is_in_selection (GSList *selection, ESource *source)
 {
@@ -156,14 +146,14 @@ update_uris_for_selection (CalendarComponent *calendar_component)
 		ESource *old_selected_source = l->data;
 
 		if (!is_in_selection (selection, old_selected_source))
-			remove_uri_for_source (old_selected_source, priv->calendar);
+			gnome_calendar_remove_event_source (priv->calendar, old_selected_source);
 	}	
 	
 	for (l = selection; l; l = l->next) {
 		ESource *selected_source = l->data;
 		
-		add_uri_for_source (selected_source, priv->calendar);
-		uids_selected = g_slist_append (uids_selected, (char *)e_source_peek_uid (selected_source));
+		gnome_calendar_add_event_source (priv->calendar, selected_source);
+		uids_selected = g_slist_append (uids_selected, (char *) e_source_peek_uid (selected_source));
 	}
 	
 	e_source_selector_free_selection (priv->source_selection);
@@ -179,19 +169,19 @@ update_uri_for_primary_selection (CalendarComponent *calendar_component)
 {
 	CalendarComponentPrivate *priv;
 	ESource *source;
-	char *uri;
 
 	priv = calendar_component->priv;
-	
+
 	source = e_source_selector_peek_primary_selection (E_SOURCE_SELECTOR (priv->source_selector));
 	if (!source)
 		return;
 
 	/* Set the default */
-	uri = e_source_get_uri (source);
-	gnome_calendar_set_default_uri (priv->calendar, uri);
-	g_free (uri);
-	
+	gnome_calendar_set_default_source (priv->calendar, source);
+
+	/* Make sure we are embedded first */
+	calendar_control_sensitize_calendar_commands (priv->view_control, priv->calendar, TRUE);
+
 	/* Save the selection for next time we start up */
 	calendar_config_set_primary_calendar (e_source_peek_uid (source));
 }
@@ -252,7 +242,7 @@ update_primary_selection (CalendarComponent *calendar_component)
 		ESource *source;
 		
 		/* Try to create a default if there isn't one */
-		source = find_first_source (priv->source_list);
+		source = e_source_list_peek_source_any (priv->source_list);
 		if (source)
 			e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	}
@@ -270,6 +260,8 @@ get_default_event (ECal *client, gboolean all_day)
 
 	comp = cal_comp_event_new_with_defaults (client);
 
+	g_return_val_if_fail (comp, NULL);
+	
 	location = calendar_config_get_timezone ();
 	zone = icaltimezone_get_builtin_timezone (location);
 
@@ -309,9 +301,10 @@ add_popup_menu_item (GtkMenu *menu, const char *label, const char *pixmap,
 		item = gtk_image_menu_item_new_with_label (label);
 
 		/* load the image */
-		image = gtk_image_new_from_stock (pixmap, GTK_ICON_SIZE_MENU);
-		if (!image)
+		if (g_file_test (pixmap, G_FILE_TEST_EXISTS))
 			image = gtk_image_new_from_file (pixmap);
+		else
+			image = gtk_image_new_from_stock (pixmap, GTK_ICON_SIZE_MENU);
 
 		if (image) {
 			gtk_widget_show (image);
@@ -343,7 +336,7 @@ copy_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 	if (!selected_source)
 		return;
 
-	copy_source_dialog (GTK_WINDOW (gtk_widget_get_toplevel (widget)), selected_source, CALOBJ_TYPE_EVENT);
+	copy_source_dialog (GTK_WINDOW (gtk_widget_get_toplevel (widget)), selected_source, E_CAL_SOURCE_TYPE_EVENT);
 }
 
 static void
@@ -384,15 +377,14 @@ delete_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 static void
 new_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 {
-	new_calendar_dialog (GTK_WINDOW (gtk_widget_get_toplevel (widget)));
+	calendar_setup_new_calendar (GTK_WINDOW (gtk_widget_get_toplevel (widget)));
 }
 
 static void
-rename_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
+edit_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 {
 	CalendarComponentPrivate *priv;
 	ESource *selected_source;
-	GtkWidget *dialog, *entry;
 
 	priv = comp->priv;
 	
@@ -400,21 +392,7 @@ rename_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 	if (!selected_source)
 		return;
 
-	/* create the dialog to prompt the user for the new name */
-	dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (widget)),
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION,
-					 GTK_BUTTONS_OK_CANCEL,
-					 _("Rename this calendar to"));
-	entry = gtk_entry_new ();
-	gtk_entry_set_text (GTK_ENTRY (entry), e_source_peek_name (selected_source));
-	gtk_widget_show (entry);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), entry, TRUE, FALSE, 6);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
-		e_source_set_name (selected_source, gtk_entry_get_text (GTK_ENTRY (entry)));
-
-	gtk_widget_destroy (dialog);
+	calendar_setup_edit_calendar (GTK_WINDOW (gtk_widget_get_toplevel (widget)), selected_source);
 }
 
 static void
@@ -425,10 +403,12 @@ fill_popup_menu_cb (ESourceSelector *selector, GtkMenu *menu, CalendarComponent
 	sensitive = e_source_selector_peek_primary_selection (E_SOURCE_SELECTOR (comp->priv->source_selector)) ?
 		TRUE : FALSE;
 
-	add_popup_menu_item (menu, _("New Calendar"), GTK_STOCK_NEW, G_CALLBACK (new_calendar_cb), comp, TRUE);
-	add_popup_menu_item (menu, _("Copy"), NULL, G_CALLBACK (copy_calendar_cb), comp, sensitive);
-	add_popup_menu_item (menu, _("Rename"), NULL, G_CALLBACK (rename_calendar_cb), comp, sensitive);
+	add_popup_menu_item (menu, _("New Calendar"), EVOLUTION_IMAGESDIR "/evolution-calendar-mini.png",
+			     G_CALLBACK (new_calendar_cb), comp, TRUE);
+	add_popup_menu_item (menu, _("Copy"), EVOLUTION_IMAGESDIR "/folder-copy-16.png",
+			     G_CALLBACK (copy_calendar_cb), comp, sensitive);
 	add_popup_menu_item (menu, _("Delete"), GTK_STOCK_DELETE, G_CALLBACK (delete_calendar_cb), comp, sensitive);
+	add_popup_menu_item (menu, _("Properties..."), NULL, G_CALLBACK (edit_calendar_cb), comp, sensitive);
 }
 
 static void
@@ -458,6 +438,25 @@ config_primary_selection_changed_cb (GConfClient *client, guint id, GConfEntry *
 	update_primary_selection (data);
 }
 
+static gboolean
+init_calendar_publishing_cb (gpointer data)
+{	
+	/* Publish if it is time to publish again */
+	e_pub_publish (FALSE);
+
+	return FALSE;
+}
+
+static void
+conf_changed_callback (GConfClient *client,
+		       unsigned int connection_id,
+		       GConfEntry *entry,
+		       void *user_data)
+{
+	/* publish config changed, so publish */
+	e_pub_publish (TRUE);
+}
+
 /* GObject methods.  */
 
 static void
@@ -514,17 +513,112 @@ impl_finalize (GObject *object)
 
 /* Evolution::Component CORBA methods.  */
 
-static void
-control_activate_cb (BonoboControl *control, gboolean activate, gpointer data)
+static CORBA_boolean
+impl_upgradeFromVersion (PortableServer_Servant servant,
+			 CORBA_short major,
+			 CORBA_short minor,
+			 CORBA_short revision,
+			 CORBA_Environment *ev)
 {
-	GnomeCalendar *gcal;
+	CalendarComponentPrivate *priv;
+	GSList *groups;
+	ESourceGroup *group;
+	ESource *source;
+	char *base_uri, *new_dir;
+	CalendarComponent *calendar_component = CALENDAR_COMPONENT (bonobo_object_from_servant (servant));
 
-	gcal = GNOME_CALENDAR (data);
+	priv = calendar_component->priv;
 
-	if (activate)
-		calendar_control_activate (control, gcal);
-	else
-		calendar_control_deactivate (control, gcal);
+	base_uri = g_build_filename (g_get_home_dir (),
+				     "/.evolution/calendar/local/OnThisComputer/",
+				     NULL);
+
+	/* create default calendars if there are no groups */
+	groups = e_source_list_peek_groups (priv->source_list);
+	if (!groups) {
+		/* create the local source group */
+		group = e_source_group_new (_("On This Computer"), base_uri);
+		e_source_list_add_group (priv->source_list, group, -1);
+
+		/* create the remote source group */
+		group = e_source_group_new (_("On The Web"), "webcal://");
+		e_source_list_add_group (priv->source_list, group, -1);
+ 		e_source_list_sync (priv->source_list, NULL);
+	}
+
+	if (major == 1 && minor <= 4) {
+		group = e_source_list_peek_group_by_name (priv->source_list, _("On This Computer"));
+
+		/* migrate calendars from 1.4 setup */
+		if (!migrate_old_calendars (group)) {
+			/* create default calendars */
+			new_dir = g_build_filename (base_uri, "Personal/", NULL);
+			if (!e_mkdir_hier (new_dir, 0700)) {
+				source = e_source_new (_("Personal"), "Personal");
+				e_source_group_add_source (group, source, -1);
+			}
+			g_free (new_dir);
+
+			new_dir = g_build_filename (base_uri, "Work/", NULL);
+			if (!e_mkdir_hier (new_dir, 0700)) {
+				source = e_source_new (_("Work"), "Work");
+				e_source_group_add_source (group, source, -1);
+			}
+			g_free (new_dir);
+		}
+
+		g_free (base_uri);
+
+		/* create the remote source group */
+		group = e_source_group_new (_("On The Web"), "webcal://");
+		e_source_list_add_group (priv->source_list, group, -1);
+
+ 		e_source_list_sync (priv->source_list, NULL);
+	}
+
+	/* create calendar for birthdays & anniversaries */
+	if ((major < 0) ||
+	    ((major == 1) && (minor < 5)) ||
+	    ((major == 1) && (minor == 5) && (revision < 2))) {		
+		group = e_source_group_new (_("Contacts"), "contacts://");
+		source = e_source_new (_("Birthdays & Anniversaries"), "/");
+		e_source_group_add_source (group, source, -1);
+		e_source_group_set_readonly (group, TRUE);
+		e_source_list_add_group (priv->source_list, group, -1);
+ 		e_source_list_sync (priv->source_list, NULL);
+	}
+
+ 	/* create calendar for birthdays & anniversaries */
+ 	if ((major < 0) ||
+ 	    ((major == 1) && (minor < 5)) ||
+	    ((major == 1) && (minor == 5) && (revision < 2))) { 		
+ 		group = e_source_group_new (_("Birthdays"), "contacts://");
+ 		source = e_source_new (_("Birthdays & Anniversaries"), "/");
+ 		e_source_group_add_source (group, source, -1);
+ 		e_source_group_set_readonly (group, TRUE);
+ 		e_source_list_add_group (priv->source_list, group, -1);
+ 		e_source_list_sync (priv->source_list, NULL);
+  	}
+
+	return CORBA_TRUE;
+}
+
+static void
+init_calendar_publishing (CalendarComponent *calendar_component)
+{
+	guint idle_id = 0;
+	CalendarComponentPrivate *priv;
+	
+	priv = calendar_component->priv;
+	
+	gconf_client_add_dir (priv->gconf_client, CALENDAR_CONFIG_PUBLISH, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
+	priv->gconf_notify_id
+		= gconf_client_notify_add (priv->gconf_client, CALENDAR_CONFIG_PUBLISH,
+					   (GConfClientNotifyFunc) conf_changed_callback, NULL,
+					   NULL, NULL);
+	
+	idle_id = g_idle_add ((GSourceFunc) init_calendar_publishing_cb, GINT_TO_POINTER (idle_id));
 }
 
 static void
@@ -539,7 +633,6 @@ impl_createControls (PortableServer_Servant servant,
 	GtkWidget *selector_scrolled_window;
 	GtkWidget *statusbar_widget;
 	BonoboControl *sidebar_control;
-	BonoboControl *view_control;
 	BonoboControl *statusbar_control;
 	guint not;
 	
@@ -562,25 +655,25 @@ impl_createControls (PortableServer_Servant servant,
 	/* Create main calendar view */
 	/* FIXME Instead of returning, we should make a control with a
 	 * label describing the problem */
-	priv->calendar = GNOME_CALENDAR (gnome_calendar_new ());
-	if (!priv->calendar) {
-		g_warning (G_STRLOC ": could not create the calendar widget!");
-		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
-		return;
-	}
-	
-	gtk_widget_show (GTK_WIDGET (priv->calendar));
-
-	view_control = bonobo_control_new (GTK_WIDGET (priv->calendar));
-	if (!view_control) {
+	priv->view_control = control_factory_new_control ();
+	if (!priv->view_control) {
 		g_warning (G_STRLOC ": could not create the control!");
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
 		return;
 	}
-	g_object_set_data (G_OBJECT (priv->calendar), "control", view_control);
 
-	g_signal_connect (view_control, "activate", G_CALLBACK (control_activate_cb), priv->calendar);
+	priv->calendar = (GnomeCalendar *) bonobo_control_get_widget (priv->view_control);
 
+	statusbar_widget = e_task_bar_new ();
+	e_activity_handler_attach_task_bar (priv->activity_handler, E_TASK_BAR (statusbar_widget));
+	gtk_widget_show (statusbar_widget);
+	statusbar_control = bonobo_control_new (statusbar_widget);
+
+	/* Initialize Calendar Publishing */
+	init_calendar_publishing (calendar_component);
+	
+	/* connect after setting the initial selections, or we'll get unwanted calls
+	   to calendar_control_sensitize_calendar_commands */
 	g_signal_connect_object (priv->source_selector, "selection_changed",
 				 G_CALLBACK (source_selection_changed_cb), 
 				 G_OBJECT (calendar_component), 0);
@@ -591,11 +684,6 @@ impl_createControls (PortableServer_Servant servant,
 				 G_CALLBACK (fill_popup_menu_cb),
 				 G_OBJECT (calendar_component), 0);
 
-	statusbar_widget = e_task_bar_new ();
-	gtk_widget_show (statusbar_widget);
-	e_activity_handler_attach_task_bar (priv->activity_handler, E_TASK_BAR (statusbar_widget));
-	statusbar_control = bonobo_control_new (statusbar_widget);
-
 	/* Load the selection from the last run */
 	update_selection (calendar_component);	
 	update_primary_selection (calendar_component);
@@ -611,7 +699,7 @@ impl_createControls (PortableServer_Servant servant,
 	
 	/* Return the controls */
 	*corba_sidebar_control = CORBA_Object_duplicate (BONOBO_OBJREF (sidebar_control), ev);
-	*corba_view_control = CORBA_Object_duplicate (BONOBO_OBJREF (view_control), ev);
+	*corba_view_control = CORBA_Object_duplicate (BONOBO_OBJREF (priv->view_control), ev);
 	*corba_statusbar_control = CORBA_Object_duplicate (BONOBO_OBJREF (statusbar_control), ev);
 }
 
@@ -622,7 +710,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 {
 	GNOME_Evolution_CreatableItemTypeList *list = GNOME_Evolution_CreatableItemTypeList__alloc ();
 
-	list->_length  = 3;
+	list->_length  = 4;
 	list->_maximum = list->_length;
 	list->_buffer  = GNOME_Evolution_CreatableItemTypeList_allocbuf (list->_length);
 
@@ -649,6 +737,13 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 	list->_buffer[2].menuShortcut = 'd';
 	list->_buffer[2].iconName = "new_all_day_event.png";
 
+	list->_buffer[3].id = CREATE_CALENDAR_ID;
+	list->_buffer[3].description = _("New calendar");
+	list->_buffer[3].menuDescription = _("C_alendar");
+	list->_buffer[3].tooltip = _("Create a new calendar");
+	list->_buffer[3].menuShortcut = 'a';
+	list->_buffer[3].iconName = "evolution-calendar-mini.png";
+
 	return list;
 }
 
@@ -696,14 +791,14 @@ setup_create_ecal (CalendarComponent *calendar_component)
 		source = e_source_list_peek_source_by_uid (priv->source_list, uid);
 		g_free (uid);
 
-		priv->create_ecal = e_cal_new (source, CALOBJ_TYPE_EVENT);
+		priv->create_ecal = auth_new_cal_from_source (source, E_CAL_SOURCE_TYPE_EVENT);
 	} 
 
 	if (!priv->create_ecal) {
 		/* Try to create a default if there isn't one */
-		source = find_first_source (priv->source_list);
+		source = e_source_list_peek_source_any (priv->source_list);
 		if (source)
-			priv->create_ecal = e_cal_new (source, CALOBJ_TYPE_EVENT);
+			priv->create_ecal = auth_new_cal_from_source (source, E_CAL_SOURCE_TYPE_EVENT);
 	}
 		
 	if (priv->create_ecal) {
@@ -752,12 +847,20 @@ impl_requestCreateItem (PortableServer_Servant servant,
 	ECalComponent *comp;
 	EventEditor *editor;
 	gboolean is_meeting = FALSE;
+	gboolean read_only = FALSE;
 	
 	priv = calendar_component->priv;
 
-	if (!setup_create_ecal (calendar_component))
+	if (!setup_create_ecal (calendar_component)) {
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
 		return;
-		
+
+	e_cal_is_read_only (priv->create_ecal, &read_only, NULL);
+	if (read_only)
+		return;
+	
+	}
+	
 	editor = event_editor_new (priv->create_ecal);
 	
 	if (strcmp (item_type_name, CREATE_EVENT_ID) == 0) {
@@ -767,6 +870,9 @@ impl_requestCreateItem (PortableServer_Servant servant,
 	} else if (strcmp (item_type_name, CREATE_MEETING_ID) == 0) {
 		comp = get_default_event (priv->create_ecal, FALSE);
 		is_meeting = TRUE;
+	} else if (strcmp (item_type_name, CREATE_CALENDAR_ID) == 0) {
+		calendar_setup_new_calendar (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (priv->calendar))));
+		return;
 	} else {
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_UnknownType);
 		return;
@@ -791,6 +897,7 @@ calendar_component_class_init (CalendarComponentClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
+	epv->upgradeFromVersion      = impl_upgradeFromVersion;
 	epv->createControls          = impl_createControls;
 	epv->_get_userCreatableItems = impl__get_userCreatableItems;
 	epv->requestCreateItem       = impl_requestCreateItem;
@@ -803,7 +910,6 @@ static void
 calendar_component_init (CalendarComponent *component)
 {
 	CalendarComponentPrivate *priv;
-	GSList *groups;
 
 	priv = g_new0 (CalendarComponentPrivate, 1);
 
@@ -820,45 +926,6 @@ calendar_component_init (CalendarComponent *component)
 
 	priv->activity_handler = e_activity_handler_new ();
 
-	/* create default calendars if there are no groups */
-	groups = e_source_list_peek_groups (priv->source_list);
-	if (!groups) {
-		ESourceGroup *group;
-		ESource *source;
-		char *base_uri, *new_dir;
-
-		/* create the local source group */
-		base_uri = g_build_filename (g_get_home_dir (),
-					     "/.evolution/calendar/local/OnThisComputer/",
-					     NULL);
-		group = e_source_group_new (_("On This Computer"), base_uri);
-		e_source_list_add_group (priv->source_list, group, -1);
-
-		/* migrate calendars from older setup */
-		if (!migrate_old_calendars (group)) {
-			/* create default calendars */
-			new_dir = g_build_filename (base_uri, "Personal/", NULL);
-			if (!e_mkdir_hier (new_dir, 0700)) {
-				source = e_source_new (_("Personal"), "Personal");
-				e_source_group_add_source (group, source, -1);
-			}
-			g_free (new_dir);
-
-			new_dir = g_build_filename (base_uri, "Work/", NULL);
-			if (!e_mkdir_hier (new_dir, 0700)) {
-				source = e_source_new (_("Work"), "Work");
-				e_source_group_add_source (group, source, -1);
-			}
-			g_free (new_dir);
-		}
-
-		g_free (base_uri);
-
-		/* create the remote source group */
-		group = e_source_group_new (_("On The Web"), "webcal://");
-		e_source_list_add_group (priv->source_list, group, -1);
-	}
-
 	component->priv = priv;
 }
 
@@ -891,7 +958,6 @@ calendar_component_peek_config_directory (CalendarComponent *component)
 	return component->priv->config_directory;
 }
 
-
 EActivityHandler *
 calendar_component_peek_activity_handler (CalendarComponent *component)
 {
