@@ -33,14 +33,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <glade/glade.h>
-#include <gal/e-table/e-cell-combo.h>
-#include <gal/e-table/e-cell-text.h>
-#include <gal/e-table/e-table-simple.h>
-#include <gal/e-table/e-table-scrolled.h>
-#include <gal/widgets/e-unicode.h>
 #include <gal/widgets/e-popup-menu.h>
-#include <gal/widgets/e-gui-utils.h>
-#include <widgets/misc/e-dateedit.h>
 #include <e-util/e-dialog-widgets.h>
 #include "../calendar-config.h"
 #include "../e-meeting-time-sel.h"
@@ -60,7 +53,7 @@ struct _SchedulePagePrivate {
 	GtkWidget *main;
 
 	/* Model */
-	EMeetingModel *model;
+	EMeetingStore *model;
 	
 	/* Selector */
 	EMeetingTimeSelector *sel;
@@ -81,8 +74,8 @@ static void schedule_page_finalize (GObject *object);
 
 static GtkWidget *schedule_page_get_widget (CompEditorPage *page);
 static void schedule_page_focus_main_widget (CompEditorPage *page);
-static void schedule_page_fill_widgets (CompEditorPage *page, CalComponent *comp);
-static gboolean schedule_page_fill_component (CompEditorPage *page, CalComponent *comp);
+static void schedule_page_fill_widgets (CompEditorPage *page, ECalComponent *comp);
+static gboolean schedule_page_fill_component (CompEditorPage *page, ECalComponent *comp);
 static void schedule_page_set_dates (CompEditorPage *page, CompEditorPageDates *dates);
 
 static void times_changed_cb (GtkWidget *widget, gpointer data);
@@ -160,11 +153,11 @@ schedule_page_finalize (GObject *object)
 		gtk_widget_unref (priv->main);
 
 	if (priv->xml) {
-		g_object_unref((priv->xml));
+		g_object_unref (priv->xml);
 		priv->xml = NULL;
 	}
 
-	g_object_unref((priv->model));
+	g_object_unref(priv->model);
 
 	g_free (priv);
 	spage->priv = NULL;
@@ -203,12 +196,11 @@ schedule_page_focus_main_widget (CompEditorPage *page)
 
 /* Set date/time */
 static void
-update_time (SchedulePage *spage, CalComponentDateTime *start_date, CalComponentDateTime *end_date) 
+update_time (SchedulePage *spage, ECalComponentDateTime *start_date, ECalComponentDateTime *end_date) 
 {
 	SchedulePagePrivate *priv;
 	struct icaltimetype start_tt, end_tt;
 	icaltimezone *start_zone = NULL, *end_zone = NULL;
-	CalClientGetStatus status;
 	gboolean all_day;
 
 	priv = spage->priv;
@@ -218,24 +210,22 @@ update_time (SchedulePage *spage, CalComponentDateTime *start_date, CalComponent
 	   first. */
 	start_zone = icaltimezone_get_builtin_timezone_from_tzid (start_date->tzid);
 	if (!start_zone) {
-		status = cal_client_get_timezone (COMP_EDITOR_PAGE (spage)->client,
-						  start_date->tzid,
-						  &start_zone);
-		/* FIXME: Handle error better. */
-		if (status != CAL_CLIENT_GET_SUCCESS)
+		if (!e_cal_get_timezone (COMP_EDITOR_PAGE (spage)->client,
+					      start_date->tzid, &start_zone, NULL)) {
+			/* FIXME: Handle error better. */
 			g_warning ("Couldn't get timezone from server: %s",
 				   start_date->tzid ? start_date->tzid : "");
+		}
 	}
 
 	end_zone = icaltimezone_get_builtin_timezone_from_tzid (end_date->tzid);
 	if (!end_zone) {
-		status = cal_client_get_timezone (COMP_EDITOR_PAGE (spage)->client,
-						  end_date->tzid,
-						  &end_zone);
-		/* FIXME: Handle error better. */
-		if (status != CAL_CLIENT_GET_SUCCESS)
-		  g_warning ("Couldn't get timezone from server: %s",
-			     end_date->tzid ? end_date->tzid : "");
+		if (!e_cal_get_timezone (COMP_EDITOR_PAGE (spage)->client,
+					      end_date->tzid, &end_zone, NULL)) {
+			/* FIXME: Handle error better. */
+			g_warning ("Couldn't get timezone from server: %s",
+				   end_date->tzid ? end_date->tzid : "");
+		}		
 	}
 
 	start_tt = *start_date->value;
@@ -251,7 +241,7 @@ update_time (SchedulePage *spage, CalComponentDateTime *start_date, CalComponent
 	if (start_zone != end_zone) {
 		icaltimezone_convert_time (&end_tt, end_zone, start_zone);
 	}
-	e_meeting_model_set_zone (priv->model, priv->zone);
+	e_meeting_store_set_zone (priv->model, priv->zone);
 	
 	all_day = (start_tt.is_date && end_tt.is_date) ? TRUE : FALSE;
 
@@ -289,11 +279,11 @@ clear_widgets (SchedulePage *spage)
 
 /* fill_widgets handler for the schedule page */
 static void
-schedule_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
+schedule_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 {
 	SchedulePage *spage;
 	SchedulePagePrivate *priv;
-	CalComponentDateTime start_date, end_date;
+	ECalComponentDateTime start_date, end_date;
 
 	spage = SCHEDULE_PAGE (page);
 	priv = spage->priv;
@@ -304,19 +294,19 @@ schedule_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 	clear_widgets (spage);
 
 	/* Start and end times */
-	cal_component_get_dtstart (comp, &start_date);
-	cal_component_get_dtend (comp, &end_date);
+	e_cal_component_get_dtstart (comp, &start_date);
+	e_cal_component_get_dtend (comp, &end_date);
 	update_time (spage, &start_date, &end_date);
 	
-	cal_component_free_datetime (&start_date);
-	cal_component_free_datetime (&end_date);
+	e_cal_component_free_datetime (&start_date);
+	e_cal_component_free_datetime (&end_date);
 	
 	priv->updating = FALSE;
 }
 
 /* fill_component handler for the schedule page */
 static gboolean
-schedule_page_fill_component (CompEditorPage *page, CalComponent *comp)
+schedule_page_fill_component (CompEditorPage *page, ECalComponent *comp)
 {
 	SchedulePage *spage;
 	SchedulePagePrivate *priv;
@@ -405,7 +395,7 @@ init_widgets (SchedulePage *spage)
  * be created.
  **/
 SchedulePage *
-schedule_page_construct (SchedulePage *spage, EMeetingModel *emm)
+schedule_page_construct (SchedulePage *spage, EMeetingStore *ems)
 {
 	SchedulePagePrivate *priv;
 	
@@ -426,11 +416,11 @@ schedule_page_construct (SchedulePage *spage, EMeetingModel *emm)
 	}
 
 	/* Model */
-	g_object_ref((emm));
-	priv->model = emm;
+	g_object_ref (ems);
+	priv->model = ems;
 	
 	/* Selector */
-	priv->sel = E_MEETING_TIME_SELECTOR (e_meeting_time_selector_new (emm));
+	priv->sel = E_MEETING_TIME_SELECTOR (e_meeting_time_selector_new (ems));
 	e_meeting_time_selector_set_working_hours (priv->sel,
 						   calendar_config_get_day_start_hour (),
 						   calendar_config_get_day_start_minute (),
@@ -457,13 +447,13 @@ schedule_page_construct (SchedulePage *spage, EMeetingModel *emm)
  * not be created.
  **/
 SchedulePage *
-schedule_page_new (EMeetingModel *emm)
+schedule_page_new (EMeetingStore *ems)
 {
 	SchedulePage *spage;
 
 	spage = g_object_new (TYPE_SCHEDULE_PAGE, NULL);
-	if (!schedule_page_construct (spage, emm)) {
-		g_object_unref((spage));
+	if (!schedule_page_construct (spage, ems)) {
+		g_object_unref (spage);
 		return NULL;
 	}
 
@@ -476,7 +466,7 @@ times_changed_cb (GtkWidget *widget, gpointer data)
 	SchedulePage *spage = data;
 	SchedulePagePrivate *priv;
 	CompEditorPageDates dates;
-	CalComponentDateTime start_dt, end_dt;
+	ECalComponentDateTime start_dt, end_dt;
 	struct icaltimetype start_tt = icaltime_null_time ();
 	struct icaltimetype end_tt = icaltime_null_time ();
 	
