@@ -47,6 +47,7 @@
 #include "calendar-config.h"
 #include "calendar-commands.h"
 #include "e-tasks.h"
+#include "e-calendar-table.h"
 #include "tasks-control.h"
 #include "evolution-shell-component-utils.h"
 
@@ -70,11 +71,9 @@ static void tasks_control_set_property		(BonoboPropertyBag	*bag,
 static void tasks_control_activate_cb		(BonoboControl		*control,
 						 gboolean		 activate,
 						 gpointer		 user_data);
-static void tasks_control_activate		(BonoboControl		*control,
-						 ETasks			*tasks);
-static void tasks_control_deactivate		(BonoboControl		*control,
-						 ETasks			*tasks);
-
+static void tasks_control_open_task_cmd		(BonoboUIComponent	*uic,
+						 gpointer		 data,
+						 const char		*path);
 static void tasks_control_new_task_cmd		(BonoboUIComponent	*uic,
 						 gpointer		 data,
 						 const char		*path);
@@ -93,7 +92,7 @@ static void tasks_control_delete_cmd		(BonoboUIComponent	*uic,
 static void tasks_control_complete_cmd		(BonoboUIComponent	*uic,
 						 gpointer		 data,
 						 const char		*path);
-static void tasks_control_expunge_cmd		(BonoboUIComponent	*uic,
+static void tasks_control_purge_cmd		(BonoboUIComponent	*uic,
 						 gpointer		 data,
 						 const char		*path);
 static void tasks_control_print_cmd		(BonoboUIComponent	*uic,
@@ -167,11 +166,13 @@ tasks_control_get_property		(BonoboPropertyBag	*bag,
 {
 	ETasks *tasks = user_data;
 	const char *uri;
+	ECalModel *model;
 
 	switch (arg_id) {
 
 	case TASKS_CONTROL_PROPERTY_URI_IDX:
-		uri = cal_client_get_uri (e_tasks_get_cal_client (tasks));
+		model = e_calendar_table_get_model (e_tasks_get_calendar_table (tasks));
+		uri = e_cal_get_uri (e_cal_model_get_default_client (model));
 		BONOBO_ARG_SET_STRING (arg, uri);
 		break;
 
@@ -194,8 +195,9 @@ tasks_control_set_property		(BonoboPropertyBag	*bag,
 	switch (arg_id) {
 
 	case TASKS_CONTROL_PROPERTY_URI_IDX:
+		/* FIXME Remove the old uri? */
 		uri = BONOBO_ARG_GET_STRING (arg);
-		if (!e_tasks_open (tasks, uri)) {
+		if (!e_tasks_add_todo_uri (tasks, uri)) {
 			char *msg;
 
 			msg = g_strdup_printf (_("Could not load the tasks in `%s'"), uri);
@@ -231,17 +233,25 @@ tasks_control_activate_cb		(BonoboControl		*control,
 /* Sensitizes the UI Component menu/toolbar commands based on the number of
  * selected tasks.
  */
-static void
-sensitize_commands (ETasks *tasks, BonoboControl *control, int n_selected)
+void
+tasks_control_sensitize_commands (BonoboControl *control, ETasks *tasks, int n_selected)
 {
 	BonoboUIComponent *uic;
-	gboolean read_only;
+	gboolean read_only = TRUE;
+	ECalModel *model;
 
 	uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
 
-	read_only = cal_client_is_read_only (e_tasks_get_cal_client (tasks));
+	if (bonobo_ui_component_get_container (uic) == CORBA_OBJECT_NIL)
+		return;
 
+	model = e_calendar_table_get_model (e_tasks_get_calendar_table (tasks));
+	e_cal_is_read_only (e_cal_model_get_default_client (model), &read_only, NULL);
+
+	bonobo_ui_component_set_prop (uic, "/commands/TasksOpenTask", "sensitive",
+				      n_selected != 1 ? "0" : "1",
+				      NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/TasksCut", "sensitive",
 				      n_selected == 0 || read_only ? "0" : "1",
 				      NULL);
@@ -249,7 +259,7 @@ sensitize_commands (ETasks *tasks, BonoboControl *control, int n_selected)
 				      n_selected == 0 ? "0" : "1",
 				      NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/TasksPaste", "sensitive",
-				      n_selected == 0 || read_only ? "0" : "1",
+				      read_only ? "0" : "1",
 				      NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/TasksDelete", "sensitive",
 				      n_selected == 0 || read_only ? "0" : "1",
@@ -257,7 +267,7 @@ sensitize_commands (ETasks *tasks, BonoboControl *control, int n_selected)
 	bonobo_ui_component_set_prop (uic, "/commands/TasksMarkComplete", "sensitive",
 				      n_selected == 0 || read_only ? "0" : "1",
 				      NULL);
-	bonobo_ui_component_set_prop (uic, "/commands/TasksExpunge", "sensitive",
+	bonobo_ui_component_set_prop (uic, "/commands/TasksPurge", "sensitive",
 				      read_only ? "0" : "1",
 				      NULL);
 }
@@ -270,39 +280,25 @@ selection_changed_cb (ETasks *tasks, int n_selected, gpointer data)
 
 	control = BONOBO_CONTROL (data);
 
-	sensitize_commands (tasks, control, n_selected);
+	tasks_control_sensitize_commands (control, tasks, n_selected);
 }
 
 static BonoboUIVerb verbs [] = {
+	BONOBO_UI_VERB ("TasksOpenTask", tasks_control_open_task_cmd),
 	BONOBO_UI_VERB ("TasksNewTask", tasks_control_new_task_cmd),
 	BONOBO_UI_VERB ("TasksCut", tasks_control_cut_cmd),
 	BONOBO_UI_VERB ("TasksCopy", tasks_control_copy_cmd),
 	BONOBO_UI_VERB ("TasksPaste", tasks_control_paste_cmd),
 	BONOBO_UI_VERB ("TasksDelete", tasks_control_delete_cmd),
 	BONOBO_UI_VERB ("TasksMarkComplete", tasks_control_complete_cmd),
-	BONOBO_UI_VERB ("TasksExpunge", tasks_control_expunge_cmd),
+	BONOBO_UI_VERB ("TasksPurge", tasks_control_purge_cmd),
 	BONOBO_UI_VERB ("TasksPrint", tasks_control_print_cmd),
 	BONOBO_UI_VERB ("TasksPrintPreview", tasks_control_print_preview_cmd),
 
 	BONOBO_UI_VERB_END
 };
 
-static EPixmap pixmaps [] = {
-	E_PIXMAP ("/menu/File/Print/Print",			    "print.xpm"),
-	E_PIXMAP ("/menu/File/Print/PrintPreview",		    "print-preview.xpm"),
-	E_PIXMAP ("/menu/EditPlaceholder/Edit/TasksCut",            "16_cut.png"),
-	E_PIXMAP ("/menu/EditPlaceholder/Edit/TasksCopy",           "16_copy.png"),
-	E_PIXMAP ("/menu/EditPlaceholder/Edit/TasksPaste",          "16_paste.png"),
-	E_PIXMAP ("/menu/EditPlaceholder/Edit/TasksDelete",         "evolution-trash-mini.png"),
-	E_PIXMAP ("/Toolbar/Cut",                                   "buttons/cut.png"),
-	E_PIXMAP ("/Toolbar/Copy",                                  "buttons/copy.png"),
-	E_PIXMAP ("/Toolbar/Paste",                                 "buttons/paste.png"),
-	E_PIXMAP ("/Toolbar/Delete",                                "buttons/delete-message.png"),
-	E_PIXMAP ("/Toolbar/Print",				    "buttons/print.png"),
-	E_PIXMAP_END
-};
-
-static void
+void
 tasks_control_activate (BonoboControl *control, ETasks *tasks)
 {
 	Bonobo_UIContainer remote_uih;
@@ -329,8 +325,6 @@ tasks_control_activate (BonoboControl *control, ETasks *tasks)
 			       "evolution-tasks",
 			       NULL);
 
-	e_pixmaps_update (uic, pixmaps);
-
 	e_tasks_setup_view_menus (tasks, uic);
 
 	/* Signals from the tasks widget; also sensitize the menu items as appropriate */
@@ -341,7 +335,7 @@ tasks_control_activate (BonoboControl *control, ETasks *tasks)
 	etable = e_calendar_table_get_table (cal_table);
 	n_selected = e_table_selected_count (etable);
 
-	sensitize_commands (tasks, control, n_selected);
+	tasks_control_sensitize_commands (control, tasks, n_selected);
 
 	bonobo_ui_component_thaw (uic, NULL);
 
@@ -351,12 +345,10 @@ tasks_control_activate (BonoboControl *control, ETasks *tasks)
 #if 0
 	calendar_config_check_timezone_set ();
 #endif
-
-	control_util_set_folder_bar_label (control, "");
 }
 
 
-static void
+void
 tasks_control_deactivate (BonoboControl *control, ETasks *tasks)
 {
 	BonoboUIComponent *uic = bonobo_control_get_ui_component (control);
@@ -374,6 +366,15 @@ tasks_control_deactivate (BonoboControl *control, ETasks *tasks)
  	bonobo_ui_component_unset_container (uic, NULL);
 }
 
+static void tasks_control_open_task_cmd		(BonoboUIComponent	*uic,
+						 gpointer		 data,
+						 const char		*path)
+{
+	ETasks *tasks;
+
+	tasks = E_TASKS (data);
+	e_tasks_open_task (tasks);
+}
 
 static void
 tasks_control_new_task_cmd		(BonoboUIComponent	*uic,
@@ -448,17 +449,17 @@ tasks_control_complete_cmd		(BonoboUIComponent	*uic,
 }
 
 static gboolean
-confirm_expunge (ETasks *tasks)
+confirm_purge (ETasks *tasks)
 {
-	GtkWidget *dialog, *label, *checkbox, *parent;
+	GtkWidget *dialog, *checkbox, *parent;
 	int button;
 	
-	if (!calendar_config_get_confirm_expunge ())
+	if (!calendar_config_get_confirm_purge ())
 		return TRUE;
 
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (tasks));
 	dialog = gtk_message_dialog_new (
-		parent,
+		(GtkWindow *)parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_WARNING,
 		GTK_BUTTONS_YES_NO,
@@ -471,22 +472,22 @@ confirm_expunge (ETasks *tasks)
 	
 	button = gtk_dialog_run (GTK_DIALOG (dialog));	
 	if (button == GTK_RESPONSE_YES && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
-		calendar_config_set_confirm_expunge (FALSE);
+		calendar_config_set_confirm_purge (FALSE);
 	gtk_widget_destroy (dialog);
 	
 	return button == GTK_RESPONSE_YES ? TRUE : FALSE;
 }
 
 static void
-tasks_control_expunge_cmd		(BonoboUIComponent	*uic,
-					 gpointer		 data,
-					 const char		*path)
+tasks_control_purge_cmd	(BonoboUIComponent	*uic,
+			 gpointer		 data,
+			 const char		*path)
 {
 	ETasks *tasks;
 
 	tasks = E_TASKS (data);
 	
-	if (confirm_expunge (tasks))
+	if (confirm_purge (tasks))
 	    e_tasks_delete_completed (tasks);
 }
 
@@ -523,7 +524,7 @@ print_tasks (ETasks *tasks, gboolean preview)
 	ETable *etable;
 	GnomePrintContext *pc;
 	GnomePrintJob *gpm;
-	double l, r, t, b, page_width, page_height, left_margin, bottom_margin, temp_d;
+	double l, r, t, b, page_width, page_height, left_margin, bottom_margin;
 
 	if (!print_config)
 		print_config = gnome_print_config_default ();
