@@ -5,7 +5,7 @@
  * Author:
  *  Bertrand Guiheneuf <bertrand@helixcode.com>
  *
- * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
+ * Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -22,16 +22,18 @@
  * USA
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <string.h>
+
 #include "camel-folder.h"
 #include "camel-exception.h"
 #include "camel-store.h"
 #include "camel-mime-message.h"
-#include "string-utils.h"
+
 #include "e-util/e-memory.h"
 #include "camel-operation.h"
 
@@ -39,6 +41,7 @@
 #include "camel-filter-driver.h"
 #include "camel-private.h"
 #include "camel-vtrash-folder.h"
+#include "filter/filter-rule.h"
 
 #define d(x) 
 #define w(x)
@@ -176,10 +179,8 @@ camel_folder_init (gpointer object, gpointer klass)
 	folder->priv = g_malloc0(sizeof(*folder->priv));
 	folder->priv->frozen = 0;
 	folder->priv->changed_frozen = camel_folder_change_info_new();
-#ifdef ENABLE_THREADS
 	folder->priv->lock = e_mutex_new(E_MUTEX_REC);
 	folder->priv->change_lock = e_mutex_new(E_MUTEX_SIMPLE);
-#endif
 }
 
 static void
@@ -193,16 +194,16 @@ camel_folder_finalize (CamelObject *object)
 	g_free(camel_folder->description);
 
 	if (camel_folder->parent_store)
-		camel_object_unref (CAMEL_OBJECT (camel_folder->parent_store));
+		camel_object_unref (camel_folder->parent_store);
 
 	if (camel_folder->summary)
-		camel_object_unref((CamelObject *)camel_folder->summary);
+		camel_object_unref (camel_folder->summary);
 
 	camel_folder_change_info_free(p->changed_frozen);
-#ifdef ENABLE_THREADS
+	
 	e_mutex_destroy(p->lock);
 	e_mutex_destroy(p->change_lock);
-#endif
+	
 	g_free(p);
 }
 
@@ -245,7 +246,7 @@ camel_folder_construct (CamelFolder *folder, CamelStore *parent_store,
 
 	folder->parent_store = parent_store;
 	if (parent_store)
-		camel_object_ref (CAMEL_OBJECT (parent_store));
+		camel_object_ref(parent_store);
 
 	folder->name = g_strdup (name);
 	folder->full_name = g_strdup (full_name);
@@ -311,7 +312,7 @@ static int
 folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 {
 	CamelFolder *folder = (CamelFolder *)object;
-	int i, count=args->argc;
+	int i;
 	guint32 tag;
 
 	for (i=0;i<args->argc;i++) {
@@ -377,18 +378,17 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 		case CAMEL_FOLDER_ARG_INFO_ARRAY:
 			*arg->ca_ptr = camel_folder_summary_array(folder->summary);
 			break;
+		case CAMEL_FOLDER_ARG_PROPERTIES:
+			*arg->ca_ptr = NULL;
+			break;
 		default:
-			count--;
 			continue;
 		}
 
 		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
 	}
 
-	if (count)
-		return parent_class->getv(object, ex, args);
-
-	return 0;
+	return parent_class->getv(object, ex, args);
 }
 
 static void
@@ -408,6 +408,9 @@ folder_free(CamelObject *o, guint32 tag, void *val)
 	case CAMEL_FOLDER_ARG_INFO_ARRAY:
 		camel_folder_summary_array_free(folder->summary, val);
 		break;
+	case CAMEL_FOLDER_ARG_PROPERTIES:
+		g_slist_free(val);
+		break;
 	default:
 		parent_class->free(o, tag, val);
 	}
@@ -547,7 +550,7 @@ get_unread_message_count(CamelFolder *folder)
 		CamelMessageInfo *info = camel_folder_summary_index(folder->summary, i);
 
 		if (info) {
-			if (!(info->flags & CAMEL_MESSAGE_SEEN))
+			if (!(info->flags & CAMEL_MESSAGE_SEEN) && (!(info->flags & CAMEL_MESSAGE_JUNK) || (folder->folder_flags & CAMEL_FOLDER_IS_JUNK)))
 				unread++;
 			camel_folder_summary_info_free(folder->summary, info);
 		}
@@ -700,7 +703,7 @@ set_message_flags(CamelFolder *folder, const char *uid, guint32 flags, guint32 s
 	camel_folder_summary_touch(folder->summary);
 	camel_folder_summary_info_free(folder->summary, info);
 
-	camel_object_trigger_event(CAMEL_OBJECT(folder), "message_changed", (char *) uid);
+	camel_object_trigger_event (folder, "message_changed", (char *) uid);
 }
 
 /**
@@ -777,7 +780,7 @@ set_message_user_flag(CamelFolder *folder, const char *uid, const char *name, gb
 	if (camel_flag_set(&info->user_flags, name, value)) {
 		info->flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
 		camel_folder_summary_touch(folder->summary);
-		camel_object_trigger_event(CAMEL_OBJECT(folder), "message_changed", (char *) uid);
+		camel_object_trigger_event (folder, "message_changed", (char *) uid);
 	}
 	camel_folder_summary_info_free(folder->summary, info);
 }
@@ -814,7 +817,7 @@ get_message_user_tag(CamelFolder *folder, const char *uid, const char *name)
 	if (info == NULL)
 		return NULL;
 
-#warning "Need to duplicate tag string"
+	/* FIXME: Need to duplicate tag string */
 
 	ret = camel_tag_get(&info->user_tags, name);
 	camel_folder_summary_info_free(folder->summary, info);
@@ -837,7 +840,7 @@ camel_folder_get_message_user_tag (CamelFolder *folder, const char *uid,  const
 
 	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), 0);
 
-#warning "get_message_user_tag() needs to copy the tag contents"
+	/* FIXME: should duplicate string */
 	ret = CF_CLASS (folder)->get_message_user_tag (folder, uid, name);
 
 	return ret;
@@ -857,7 +860,7 @@ set_message_user_tag(CamelFolder *folder, const char *uid, const char *name, con
 	if (camel_tag_set(&info->user_tags, name, value)) {
 		info->flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
 		camel_folder_summary_touch(folder->summary);
-		camel_object_trigger_event(CAMEL_OBJECT(folder), "message_changed", (char *) uid);
+		camel_object_trigger_event (folder, "message_changed", (char *) uid);
 	}
 	camel_folder_summary_info_free(folder->summary, info);
 }
@@ -1014,7 +1017,7 @@ static GPtrArray *
 get_uids(CamelFolder *folder)
 {
 	GPtrArray *array;
-	int i, count;
+	int i, j, count;
 
 	array = g_ptr_array_new();
 
@@ -1022,17 +1025,17 @@ get_uids(CamelFolder *folder)
 
 	count = camel_folder_summary_count(folder->summary);
 	g_ptr_array_set_size(array, count);
-	for (i=0; i<count; i++) {
+	for (i = 0, j = 0; i < count; i++) {
 		CamelMessageInfo *info = camel_folder_summary_index(folder->summary, i);
-
+		
 		if (info) {
-			array->pdata[i] = g_strdup(camel_message_info_uid(info));
+			array->pdata[j++] = g_strdup (camel_message_info_uid (info));
 			camel_folder_summary_info_free(folder->summary, info);
-		} else {
-			array->pdata[i] = g_strdup("xx unknown uid xx");
 		}
 	}
-
+	
+	g_ptr_array_set_size (array, j);
+	
 	return array;
 }
 
@@ -1287,7 +1290,7 @@ transfer_message_to (CamelFolder *source, const char *uid, CamelFolder *dest,
 	}
 	
 	camel_folder_append_message (dest, msg, info, transferred_uid, ex);
-	camel_object_unref (CAMEL_OBJECT (msg));
+	camel_object_unref (msg);
 	
 	if (delete_original && !camel_exception_is_set (ex))
 		camel_folder_set_message_flags (source, uid, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN, ~0);
@@ -1413,7 +1416,7 @@ camel_folder_delete (CamelFolder *folder)
 
 	CAMEL_FOLDER_UNLOCK (folder, lock);
 
-	camel_object_trigger_event (CAMEL_OBJECT (folder), "deleted", NULL);
+	camel_object_trigger_event (folder, "deleted", NULL);
 }
 
 static void
@@ -1449,7 +1452,7 @@ camel_folder_rename(CamelFolder *folder, const char *new)
 
 	CF_CLASS (folder)->rename(folder, new);
 
-	camel_object_trigger_event (CAMEL_OBJECT (folder), "renamed", old);
+	camel_object_trigger_event (folder, "renamed", old);
 	g_free(old);
 }
 
@@ -1505,7 +1508,7 @@ thaw (CamelFolder * folder)
 	CAMEL_FOLDER_UNLOCK(folder, change_lock);
 
 	if (info) {
-		camel_object_trigger_event(CAMEL_OBJECT(folder), "folder_changed", info);
+		camel_object_trigger_event (folder, "folder_changed", info);
 		camel_folder_change_info_free(info);
 	}
 }
@@ -1546,7 +1549,7 @@ camel_folder_is_frozen (CamelFolder *folder)
 	return CF_CLASS (folder)->is_frozen (folder);
 }
 
-#ifdef ENABLE_THREADS
+
 struct _folder_filter_msg {
 	CamelSessionThreadMsg msg;
 
@@ -1560,9 +1563,52 @@ static void
 filter_filter(CamelSession *session, CamelSessionThreadMsg *msg)
 {
 	struct _folder_filter_msg *m = (struct _folder_filter_msg *)msg;
-	
-	camel_filter_driver_filter_folder(m->driver, m->folder, NULL, m->recents, FALSE, &m->ex);
-	camel_filter_driver_flush (m->driver, &m->ex);
+	CamelMessageInfo *info;
+	int i, status = 0;
+	CamelURL *uri;
+	char *source_url;
+	CamelException ex;
+
+	camel_operation_start(NULL, _("Filtering new message(s)"));
+
+	source_url = camel_service_get_url((CamelService *)m->folder->parent_store);
+	uri = camel_url_new(source_url, NULL);
+	g_free(source_url);
+	if (m->folder->full_name && m->folder->full_name[0] != '/') {
+		char *tmp = alloca(strlen(m->folder->full_name)+2);
+
+		sprintf(tmp, "/%s", m->folder->full_name);
+		camel_url_set_path(uri, tmp);
+	} else
+		camel_url_set_path(uri, m->folder->full_name);
+	source_url = camel_url_to_string(uri, CAMEL_URL_HIDE_ALL);
+	camel_url_free(uri);
+
+	for (i=0;status == 0 && i<m->recents->len;i++) {
+		char *uid = m->recents->pdata[i];
+		int pc = 100 * i / m->recents->len;
+
+		camel_operation_progress(NULL, pc);
+
+		info = camel_folder_get_message_info(m->folder, uid);
+		if (info == NULL) {
+			g_warning("uid %s vanished from folder: %s", uid, source_url);
+			continue;
+		}
+
+		status = camel_filter_driver_filter_message(m->driver, NULL, info, uid, m->folder, source_url, source_url, &m->ex);
+
+		camel_folder_free_message_info(m->folder, info);
+	}
+
+	camel_exception_init(&ex);
+	camel_filter_driver_flush(m->driver, &ex);
+	if (!camel_exception_is_set(&m->ex))
+		camel_exception_xfer(&m->ex, &ex);
+
+	g_free(source_url);
+
+	camel_operation_end(NULL);
 }
 
 static void
@@ -1583,7 +1629,7 @@ static CamelSessionThreadOps filter_ops = {
 	filter_filter,
 	filter_free,
 };
-#endif
+
 
 /* Event hooks that block emission when frozen */
 static gboolean
@@ -1602,14 +1648,14 @@ folder_changed (CamelObject *obj, gpointer event_data)
 		CamelSession *session = ((CamelService *)folder->parent_store)->session;
 		CamelFilterDriver *driver = NULL;
 
-		if ((folder->folder_flags & CAMEL_FOLDER_FILTER_RECENT)
+		if (((folder->folder_flags & CAMEL_FOLDER_FILTER_RECENT) || camel_session_check_junk (session))
 		    && changed->uid_recent->len > 0)
-			driver = camel_session_get_filter_driver(session, "incoming", NULL);
-			
+			driver = camel_session_get_filter_driver(session,
+								 (folder->folder_flags & CAMEL_FOLDER_FILTER_RECENT) ? FILTER_SOURCE_INCOMING : FILTER_SOURCE_JUNKTEST, NULL);
+		
 		CAMEL_FOLDER_LOCK(folder, change_lock);
 
 		if (driver) {
-#ifdef ENABLE_THREADS
 			GPtrArray *recents = g_ptr_array_new();
 			int i;
 			struct _folder_filter_msg *msg;
@@ -1626,13 +1672,7 @@ folder_changed (CamelObject *obj, gpointer event_data)
 			msg->driver = driver;
 			camel_exception_init(&msg->ex);
 			camel_session_thread_queue(session, &msg->msg, 0);
-#else
-			d(printf("Have '%d' recent messages, filtering\n", changed->recent->len));
-			folder->priv->frozen++;
-			camel_filter_driver_filter_folder(driver, folder, NULL, changed->recent, FALSE, NULL);
-			camel_object_unref((CamelObject *)driver);
-			folder->priv->frozen--;
-#endif
+			
 			/* zero out the recent list so we dont reprocess */
 			/* this pokes past abstraction, but changeinfo is our structure anyway */
 			/* the only other alternative is to recognise when trigger is called from	
