@@ -33,7 +33,6 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <camel/camel-cipher-context.h>
-#include <camel/camel-cms-context.h>
 #include <camel/camel-data-wrapper.h>
 #include <camel/camel-exception.h>
 #include <camel/camel-folder.h>
@@ -64,14 +63,11 @@ extern "C" {
 #include <camel/camel-multipart-encrypted.h>
 #include <camel/camel-multipart-signed.h>
 #include <camel/camel-gpg-context.h>
-#include <camel/camel-pgp-mime.h>
 #include <camel/camel-provider.h>
 #include <camel/camel-seekable-stream.h>
 #include <camel/camel-seekable-substream.h>
 #include <camel/camel-service.h>
 #include <camel/camel-session.h>
-#include <camel/camel-smime-context.h>
-#include <camel/camel-smime-utils.h>
 #include <camel/camel-store.h>
 #include <camel/camel-stream-buffer.h>
 #include <camel/camel-stream-filter.h>
@@ -84,7 +80,7 @@ extern "C" {
 #include <camel/camel-transport.h>
 #include <camel/camel-uid-cache.h>
 #include <camel/camel-url.h>
-#include <camel/string-utils.h>
+#include <camel/camel-string-utils.h>
 
 gint camel_init (const char *certdb_dir, gboolean nss_init);
 
