@@ -31,10 +31,10 @@
 #ifdef __cplusplus
 extern "C" {
 #pragma }
-#endif /* __cplusplus }*/
+#endif /* __cplusplus */
 
 #include "camel-imap-types.h"
-#include "camel-disco-folder.h"
+#include <camel/camel-disco-folder.h>
 #include <camel/camel-folder-search.h>
 
 #define CAMEL_IMAP_FOLDER_TYPE     (camel_imap_folder_get_type ())
@@ -47,11 +47,12 @@ struct _CamelImapFolder {
 
 	struct _CamelImapFolderPrivate *priv;
 
-	gboolean need_rescan, need_refresh;
 	CamelFolderSearch *search;
 	CamelImapMessageCache *cache;
-};
 
+	unsigned int need_rescan:1;
+	unsigned int need_refresh:1;
+};
 
 typedef struct {
 	CamelDiscoFolderClass parent_class;
