@@ -3,9 +3,8 @@
  * Copyright (C) 2001 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,4 +32,5 @@ gboolean e_path_find_folders (const char *prefix,
 			      EPathFindFoldersCallback callback,
 			      gpointer data);
 
+int      e_path_rmdir        (const char *prefix, const char *vpath);
 #endif /* __E_PATH__ */
