@@ -39,6 +39,7 @@
 
 #include "mail-vfolder.h"
 #include "mail-autofilter.h"
+#include "mail-component.h"
 
 #include "camel/camel.h"
 
@@ -51,7 +52,6 @@
 #include "filter/filter-editor.h"
 #include "filter/filter-option.h"
 
-extern char *evolution_dir;
 
 static void
 rule_match_recipients (RuleContext *context, FilterRule *rule, CamelInternetAddress *iaddr)
@@ -222,6 +222,28 @@ rule_match_mlist(RuleContext *context, FilterRule *rule, const char *mlist)
 	filter_input_set_value((FilterInput *)element, mlist);
 }
 
+static void
+rule_match_thread (RuleContext *context, FilterRule *rule, const char *msgid)
+{
+	FilterElement *element;
+	FilterPart *part;
+	
+	if (msgid[0] == 0)
+		return;
+	
+	part = rule_context_create_part (context, "header");
+	filter_rule_add_part (rule, part);
+	
+	element = filter_part_find_element (part, "header-type");
+	filter_option_set_current ((FilterOption *) element, "contains");
+
+	element = filter_part_find_element (part, "header-field");
+	filter_input_set_value ((FilterInput *) element, "References");
+	
+	element = filter_part_find_element (part, "word");
+	filter_input_set_value ((FilterInput *) element, msgid);
+}
+
 static void
 rule_from_message (FilterRule *rule, RuleContext *context, CamelMimeMessage *msg, int flags)
 {
@@ -265,7 +287,7 @@ rule_from_message (FilterRule *rule, RuleContext *context, CamelMimeMessage *msg
 	if (flags & AUTO_MLIST) {
 		char *name, *mlist;
 		
-		mlist = header_raw_check_mailing_list(&((CamelMimePart *)msg)->headers);
+		mlist = camel_header_raw_check_mailing_list (&((CamelMimePart *)msg)->headers);
 		if (mlist) {
 			rule_match_mlist(context, rule, mlist);
 			name = g_strdup_printf (_("%s mailing list"), mlist);
@@ -274,6 +296,30 @@ rule_from_message (FilterRule *rule, RuleContext *context, CamelMimeMessage *msg
 		}
 		g_free(mlist);
 	}
+	if (flags & AUTO_THREAD) {
+		const char *msgid = NULL, *refs = NULL;
+		char *name;
+		
+		if (!(refs = camel_medium_get_header ((CamelMedium *) msg, "References"))) {
+			if (!(refs = camel_medium_get_header ((CamelMedium *) msg, "In-Reply-To")))
+				msgid = camel_mime_message_get_message_id (msg);
+		}
+		
+		if (refs || msgid) {
+			struct _camel_header_references *r = NULL;
+			
+			if (refs) {
+				r = camel_header_references_decode (refs);
+				msgid = r->id;
+			}
+			
+			rule_match_thread (context, rule, msgid);
+			name = g_strdup_printf (_("Replies to %s"), msgid);
+			
+			if (r)
+				camel_header_references_list_clear (&r);
+		}
+	}
 }
 
 FilterRule *
@@ -313,7 +359,8 @@ filter_gui_add_from_message (CamelMimeMessage *msg, const char *source, int flag
 	g_return_if_fail (msg != NULL);
 	
 	fc = filter_context_new ();
-	user = g_strdup_printf ("%s/filters.xml", evolution_dir);
+	user = g_strdup_printf ("%s/mail/filters.xml",
+				mail_component_peek_base_directory (mail_component_peek ()));
 	system = EVOLUTION_PRIVDATADIR "/filtertypes.xml";
 	rule_context_load ((RuleContext *)fc, system, user);
 	rule = filter_rule_from_message (fc, msg, flags);
@@ -334,7 +381,7 @@ mail_filter_rename_uri(CamelStore *store, const char *olduri, const char *newuri
 	GList *changed;
 	
 	fc = filter_context_new ();
-	user = g_strdup_printf ("%s/filters.xml", evolution_dir);
+	user = g_strdup_printf ("%s/mail/filters.xml", mail_component_peek_base_directory (mail_component_peek ()));
 	system = EVOLUTION_PRIVDATADIR "/filtertypes.xml";
 	rule_context_load ((RuleContext *)fc, system, user);
 	
@@ -359,7 +406,7 @@ mail_filter_delete_uri(CamelStore *store, const char *uri)
 	GList *deleted;
 	
 	fc = filter_context_new ();
-	user = g_strdup_printf ("%s/filters.xml", evolution_dir);
+	user = g_strdup_printf ("%s/mail/filters.xml", mail_component_peek_base_directory (mail_component_peek ()));
 	system = EVOLUTION_PRIVDATADIR "/filtertypes.xml";
 	rule_context_load ((RuleContext *)fc, system, user);
 	
