@@ -162,8 +162,8 @@ static void
 setup_search_context (MailComponent *component)
 {
 	MailComponentPrivate *priv = component->priv;
-	char *user = g_strdup_printf ("%s/evolution/searches.xml", g_get_home_dir ()); /* EPFIXME should be somewhere else. */
-	char *system = g_strdup (EVOLUTION_PRIVDATADIR "/vfoldertypes.xml");
+	char *user = g_build_filename(component->priv->base_directory, "mail/searches.xml", NULL);
+	char *system = g_strdup (EVOLUTION_PRIVDATADIR "/searchtypes.xml");
 	
 	priv->search_context = rule_context_new ();
 	g_object_set_data_full (G_OBJECT (priv->search_context), "user", user, g_free);
@@ -235,6 +235,55 @@ folder_selected_cb (EMFolderTree *emft, const char *path, const char *uri, EMFol
 		em_folder_view_set_folder_uri (view, uri);
 }
 
+#define PROPERTY_FOLDER_URI          "folder_uri"
+#define PROPERTY_FOLDER_URI_IDX      1
+
+static void
+set_prop (BonoboPropertyBag *bag,
+	  const BonoboArg   *arg,
+	  guint              arg_id,
+	  CORBA_Environment *ev,
+	  gpointer           user_data)
+{
+	EMFolderView *view  = (EMFolderView *)bonobo_control_get_widget (user_data);
+	const gchar *uri;
+
+	switch (arg_id) {
+	case PROPERTY_FOLDER_URI_IDX:
+		uri = BONOBO_ARG_GET_STRING (arg);
+		
+		g_warning ("XXX setting uri blah=\"%s\"\n", uri);
+
+		em_folder_view_set_folder_uri (view, uri);
+		break;
+	default:
+		g_warning ("Unhandled arg %d\n", arg_id);
+		break;
+	}
+}
+
+static void
+get_prop (BonoboPropertyBag *bag,
+	  BonoboArg         *arg,
+	  guint              arg_id,
+	  CORBA_Environment *ev,
+	  gpointer           user_data)
+{
+	GtkWidget *widget = bonobo_control_get_widget (user_data);
+	EMFolderView *view = (EMFolderView *)widget;
+
+	switch (arg_id) {
+	case PROPERTY_FOLDER_URI_IDX:
+		if (view->folder_uri)
+			BONOBO_ARG_SET_STRING (arg, view->folder_uri);
+		else 
+			BONOBO_ARG_SET_STRING (arg, "");
+		break;
+	default:
+		g_warning ("Unhandled arg %d\n", arg_id);
+	}
+}
+
 static void
 view_control_activate_cb (BonoboControl *control, gboolean activate, EMFolderView *view)
 {
@@ -319,6 +368,13 @@ impl_finalize (GObject *object)
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static void
+view_on_url (GObject *emitter, const char *url, const char *nice_url, MailComponent *mail_component)
+{
+	MailComponentPrivate *priv = mail_component->priv;
+	
+	e_activity_handler_set_message (priv->activity_handler, nice_url);
+}
 
 /* Evolution::Component CORBA methods.  */
 
@@ -342,6 +398,9 @@ impl_createControls (PortableServer_Servant servant,
 	tree_widget = (GtkWidget *) em_folder_tree_new_with_model (priv->model);
 	em_folder_tree_enable_drag_and_drop ((EMFolderTree *) tree_widget);
 	em_format_set_session ((EMFormat *) ((EMFolderView *) view_widget)->preview, session);
+
+	g_signal_connect (view_widget, "on-url", G_CALLBACK (view_on_url), mail_component);
+	em_folder_view_set_statusbar ((EMFolderView*)view_widget, FALSE);
 	
 	statusbar_widget = e_task_bar_new ();
 	e_activity_handler_attach_task_bar (priv->activity_handler, E_TASK_BAR (statusbar_widget));
@@ -395,6 +454,9 @@ impl_requestCreateItem (PortableServer_Servant servant,
 				     ex_GNOME_Evolution_Component_UnknownType, NULL);
 		return;
 	}
+
+	if (!em_utils_check_user_can_send_mail(NULL))
+		return;
 	
 	em_utils_compose_new_message ();
 }
@@ -403,6 +465,9 @@ static void
 impl_handleURI (PortableServer_Servant servant, const char *uri, CORBA_Environment *ev)
 {
 	if (!strncmp (uri, "mailto:", 7)) {
+		if (!em_utils_check_user_can_send_mail(NULL))
+			return;
+
 		em_utils_compose_new_message_with_mailto (uri);
 	}
 }
@@ -463,8 +528,6 @@ mail_component_init (MailComponent *component)
 {
 	MailComponentPrivate *priv;
 	EAccountList *accounts;
-	struct stat st;
-	char *mail_dir;
 	
 	priv = g_new0 (MailComponentPrivate, 1);
 	component->priv = priv;
@@ -483,27 +546,6 @@ mail_component_init (MailComponent *component)
 	priv->async_event = mail_async_event_new();
 	priv->store_hash = g_hash_table_new (NULL, NULL);
 	
-	/* migrate evolution 1.x folders to 2.0's location/format */
-	mail_dir = g_strdup_printf ("%s/mail", priv->base_directory);
-	if (stat (mail_dir, &st) == -1) {
-		CamelException ex;
-		
-		camel_exception_init (&ex);
-		if (em_migrate (component, &ex) == -1) {
-			GtkWidget *dialog;
-			
-			dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
-							 _("The following error occured while migrating your mail data:\n%s"),
-							 camel_exception_get_description (&ex));
-			
-			g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
-			gtk_widget_show (dialog);
-			
-			camel_exception_clear (&ex);
-		}
-	}
-	g_free (mail_dir);
-	
 	setup_local_store (component);
 	
 	accounts = mail_config_get_accounts ();
@@ -519,6 +561,36 @@ mail_component_init (MailComponent *component)
 
 
 /* Public API.  */
+BonoboControl *
+mail_control_new (void) 
+{
+	BonoboControl *view_control;
+	GtkWidget *view_widget;
+	BonoboPropertyBag *pbag;
+
+	view_widget = em_folder_browser_new ();
+	gtk_widget_show (view_widget);
+
+	view_control = bonobo_control_new (view_widget);
+	pbag = bonobo_property_bag_new (get_prop, set_prop, view_control);
+  
+	bonobo_property_bag_add (pbag,
+				 PROPERTY_FOLDER_URI, 
+				 PROPERTY_FOLDER_URI_IDX,
+				 BONOBO_ARG_STRING,
+				 NULL,
+				 _("URI of the mail source that the view will display"),
+				 0);
+	
+	bonobo_control_set_properties (view_control,
+				       bonobo_object_corba_objref (BONOBO_OBJECT (pbag)),
+				       NULL);
+	bonobo_object_unref (BONOBO_OBJECT (pbag));
+	
+	g_signal_connect (view_control, "activate", G_CALLBACK (view_control_activate_cb), view_widget);
+	
+	return view_control;
+}
 
 MailComponent *
 mail_component_peek (void)
