@@ -38,18 +38,20 @@
 #include <camel/camel-vtrash-folder.h>
 #include <camel/camel-vee-store.h>
 #include "mail.h"
+#include "mail-component.h"
 #include "mail-tools.h"
 #include "mail-ops.h"
 #include "mail-vfolder.h"
 #include "mail-session.h"
 #include "composer/e-msg-composer.h"
-#include "folder-browser.h"
 
 #include "filter/filter-filter.h"
 
 #include "mail-mt.h"
 #include "mail-folder-cache.h"
 
+#include "em-utils.h"
+
 #define w(x)
 #define d(x) 
 
@@ -150,17 +152,12 @@ static void
 filter_folder_free (struct _mail_msg *mm)
 {
 	struct _filter_mail_msg *m = (struct _filter_mail_msg *)mm;
-	int i;
 	
 	if (m->source_folder)
 		camel_object_unref (m->source_folder);
 	
-	if (m->source_uids) {
-		for (i = 0; i < m->source_uids->len; i++)
-			g_free (m->source_uids->pdata[i]);
-		
-		g_ptr_array_free (m->source_uids, TRUE);
-	}
+	if (m->source_uids)
+		em_utils_uids_free (m->source_uids);
 	
 	if (m->cancel)
 		camel_operation_unref (m->cancel);
@@ -224,35 +221,17 @@ static char *
 uid_cachename_hack (CamelStore *store)
 {
 	CamelURL *url = CAMEL_SERVICE (store)->url;
-	char *encoded_url, *filename, *old_location;
-	struct stat st;
+	char *encoded_url, *filename;
+	const char *evolution_dir;
 	
-	encoded_url = g_strdup_printf ("pop://%s%s%s@%s/", url->user,
+	encoded_url = g_strdup_printf ("%s%s%s@%s", url->user,
 				       url->authmech ? ";auth=" : "",
 				       url->authmech ? url->authmech : "",
 				       url->host);
 	e_filename_make_safe (encoded_url);
 	
-	filename = g_strdup_printf ("%s/mail/pop3/cache-%s", evolution_dir, encoded_url);
-	
-	/* lame hack, but we can't expect user's to actually migrate
-           their cache files - brain power requirements are too
-           high. */
-	if (stat (filename, &st) == -1) {
-		/* This is either the first time the user has checked
-                   mail with this POP provider or else their cache
-                   file is in the old location... */
-		old_location = g_strdup_printf ("%s/config/cache-%s", evolution_dir, encoded_url);
-		if (stat (old_location, &st) == -1) {
-			/* old location doesn't exist either so use the new location */
-			g_free (old_location);
-		} else {
-			/* old location exists, so I guess we use the old cache file location */
-			g_free (filename);
-			filename = old_location;
-		}
-	}
-	
+	evolution_dir = mail_component_peek_base_directory (mail_component_peek ());
+	filename = g_build_filename (evolution_dir, "mail", "pop", encoded_url, "uid-cache", NULL);
 	g_free (encoded_url);
 	
 	return filename;
@@ -270,15 +249,12 @@ fetch_mail_fetch (struct _mail_msg *mm)
 	struct _fetch_mail_msg *m = (struct _fetch_mail_msg *)mm;
 	struct _filter_mail_msg *fm = (struct _filter_mail_msg *)mm;
 	int i;
-	
+
 	if (m->cancel)
 		camel_operation_register (m->cancel);
 	
-	if ((fm->destination = mail_tool_get_local_inbox (&mm->ex)) == NULL) {
-		if (m->cancel)
-			camel_operation_unregister (m->cancel);
-		return;
-	}
+	if ((fm->destination = mail_tool_get_local_inbox (&mm->ex)) == NULL)
+		goto fail;
 	
 	/* FIXME: this should support keep_on_server too, which would then perform a spool
 	   access thingy, right?  problem is matching raw messages to uid's etc. */
@@ -324,10 +300,8 @@ fetch_mail_fetch (struct _mail_msg *mm)
 					fm->cache = cache;
 					filter_folder_filter (mm);
 					
-					/* if we are not to delete the messages or there was an
-					 * exception, save the UID cache */
-					if (!fm->delete || camel_exception_is_set (&mm->ex))
-						camel_uid_cache_save (cache);
+					/* save the cache of uids that we've just downloaded */
+					camel_uid_cache_save (cache);
 					
 					/* if we are deleting off the server and no exception occured
 					 * then iterate through the folder uids and mark them all
@@ -357,7 +331,7 @@ fetch_mail_fetch (struct _mail_msg *mm)
 			fm->source_folder = NULL;
 		}
 	}
-	
+fail:	
 	if (m->cancel)
 		camel_operation_unregister (m->cancel);
 	
@@ -374,7 +348,7 @@ static void
 fetch_mail_fetched (struct _mail_msg *mm)
 {
 	struct _fetch_mail_msg *m = (struct _fetch_mail_msg *)mm;
-	
+
 	if (m->done)
 		m->done (m->source_uri, m->data);
 }
@@ -387,7 +361,7 @@ fetch_mail_free (struct _mail_msg *mm)
 	g_free (m->source_uri);
 	if (m->cancel)
 		camel_operation_unref (m->cancel);
-	
+
 	filter_folder_free (mm);
 }
 
@@ -419,7 +393,7 @@ mail_fetch_mail (const char *source, int keep, const char *type, CamelOperation
 	}
 	m->done = done;
 	m->data = data;
-	
+
 	fm->driver = camel_session_get_filter_driver (session, type, NULL);
 	camel_filter_driver_set_folder_func (fm->driver, get_folder, get_data);
 	if (status)
@@ -913,6 +887,10 @@ mail_append_mail (CamelFolder *folder, CamelMimeMessage *message, CamelMessageIn
 	g_assert(CAMEL_IS_FOLDER (folder));
 	g_assert(CAMEL_IS_MIME_MESSAGE (message));
 	
+	if (!camel_medium_get_header (CAMEL_MEDIUM (message), "X-Mailer"))
+		camel_medium_set_header (CAMEL_MEDIUM (message), "X-Mailer",
+					 "Ximian Evolution " VERSION SUB_VERSION " " VERSION_COMMENT);
+
 	m = mail_msg_new (&append_mail_op, NULL, sizeof (*m));
 	m->folder = folder;
 	camel_object_ref(folder);
@@ -1001,14 +979,10 @@ static void
 transfer_messages_free (struct _mail_msg *mm)
 {
 	struct _transfer_msg *m = (struct _transfer_msg *)mm;
-	int i;
-
+	
 	camel_object_unref (m->source);
 	g_free (m->dest_uri);
-	for (i = 0; i < m->uids->len; i++)
-		g_free (m->uids->pdata[i]);
-	g_ptr_array_free (m->uids, TRUE);
-
+	em_utils_uids_free (m->uids);
 }
 
 static struct _mail_msg_op transfer_messages_op = {
@@ -1069,9 +1043,9 @@ get_folderinfo_desc (struct _mail_msg *mm, int done)
 }
 
 static void
-add_vtrash_info (CamelStore *store, CamelFolderInfo *info)
+add_vtrash_or_vjunk_info (CamelStore *store, CamelFolderInfo *info, gchar *name, gchar *full_name, gchar *url_base, gboolean unread_count)
 {
-	CamelFolderInfo *fi, *vtrash, *parent;
+	CamelFolderInfo *fi, *vinfo, *parent;
 	char *uri, *path;
 	CamelURL *url;
 	
@@ -1079,14 +1053,14 @@ add_vtrash_info (CamelStore *store, CamelFolderInfo *info)
 
 	parent = NULL;
 	for (fi = info; fi; fi = fi->sibling) {
-		if (!strcmp (fi->name, CAMEL_VTRASH_NAME))
+		if (!strcmp (fi->name, name))
 			break;
 		parent = fi;
 	}
 	
-	/* create our vTrash URL */
+	/* create our vTrash/vJunk URL */
 	url = camel_url_new (info->url, NULL);
-	path = g_strdup_printf ("/%s", CAMEL_VTRASH_NAME);
+	path = g_strdup_printf ("/%s", name);
 	if (url->fragment)
 		camel_url_set_fragment (url, path);
 	else
@@ -1096,31 +1070,44 @@ add_vtrash_info (CamelStore *store, CamelFolderInfo *info)
 	camel_url_free (url);
 	
 	if (fi) {
-		/* We're going to replace the physical Trash folder with our vTrash folder */
-		vtrash = fi;
-		g_free (vtrash->full_name);
-		g_free (vtrash->name);
-		g_free (vtrash->url);
+		/* We're going to replace the physical Trash/Junk folder with our vTrash/vJunk folder */
+		vinfo = fi;
+		g_free (vinfo->full_name);
+		g_free (vinfo->name);
+		g_free (vinfo->url);
 	} else {
-		/* There wasn't a Trash folder so create a new folder entry */
-		vtrash = g_new0 (CamelFolderInfo, 1);
+		/* There wasn't a Trash/Junk folder so create a new folder entry */
+		vinfo = g_new0 (CamelFolderInfo, 1);
 
 		g_assert(parent != NULL);
 
 		/* link it into the right spot */
-		vtrash->sibling = parent->sibling;
-		parent->sibling = vtrash;
+		vinfo->sibling = parent->sibling;
+		parent->sibling = vinfo;
 	}
 	
 	/* Fill in the new fields */
-	vtrash->full_name = g_strdup (_("Trash"));
-	vtrash->name = g_strdup(vtrash->full_name);
-	vtrash->url = g_strdup_printf ("vtrash:%s", uri);
-	vtrash->unread_message_count = -1;
-	vtrash->path = g_strdup_printf("/%s", vtrash->name);
+	vinfo->full_name = g_strdup (full_name);
+	vinfo->name = g_strdup(vinfo->full_name);
+	vinfo->url = g_strdup_printf ("%s:%s", url_base, uri);
+	if (!unread_count)
+		vinfo->unread_message_count = -1;
+	vinfo->path = g_strdup_printf("/%s", vinfo->name);
 	g_free (uri);
 }
 
+static void
+add_vtrash_info (CamelStore *store, CamelFolderInfo *info)
+{
+	add_vtrash_or_vjunk_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), "vtrash", FALSE);
+}
+
+static void
+add_vjunk_info (CamelStore *store, CamelFolderInfo *info)
+{
+	add_vtrash_or_vjunk_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), "vjunk", TRUE);
+}
+
 static void
 add_unmatched_info(CamelFolderInfo *fi)
 {
@@ -1148,6 +1135,8 @@ get_folderinfo_get (struct _mail_msg *mm)
 	if (m->info) {
 		if (m->info->url && (m->store->flags & CAMEL_STORE_VTRASH))
 			add_vtrash_info(m->store, m->info);
+		if (m->info->url && (m->store->flags & CAMEL_STORE_VJUNK))
+			add_vjunk_info(m->store, m->info);
 		if (CAMEL_IS_VEE_STORE(m->store))
 			add_unmatched_info(m->info);
 	}
@@ -1683,15 +1672,23 @@ static char *empty_trash_desc(struct _mail_msg *mm, int done)
 static void empty_trash_empty(struct _mail_msg *mm)
 {
 	struct _empty_trash_msg *m = (struct _empty_trash_msg *)mm;
+	const char *evolution_dir;
 	CamelFolder *trash;
-
-	if (m->account)
+	char *uri;
+	
+	if (m->account) {
 		trash = mail_tool_get_trash (m->account->source->url, FALSE, &mm->ex);
-	else
-		trash = mail_tool_get_trash ("file:/", TRUE, &mm->ex);
+	} else {
+		evolution_dir = mail_component_peek_base_directory (mail_component_peek ());
+		uri = g_strdup_printf ("mbox:%s/mail/local", evolution_dir);
+		trash = mail_tool_get_trash (uri, TRUE, &mm->ex);
+		g_free (uri);
+	}
+	
 	if (trash)
 		camel_folder_expunge (trash, &mm->ex);
-	camel_object_unref(trash);
+	
+	camel_object_unref (trash);
 }
 
 static void empty_trash_emptied(struct _mail_msg *mm)
@@ -1854,10 +1851,8 @@ static void get_messages_free(struct _mail_msg *mm)
 {
 	struct _get_messages_msg *m = (struct _get_messages_msg *)mm;
 	int i;
-
-	for (i=0;i<m->uids->len;i++)
-		g_free(m->uids->pdata[i]);
-	g_ptr_array_free(m->uids, TRUE);
+	
+	em_utils_uids_free (m->uids);
 	for (i=0;i<m->messages->len;i++) {
 		if (m->messages->pdata[i])
 			camel_object_unref(m->messages->pdata[i]);
@@ -1936,8 +1931,8 @@ save_prepare_part (CamelMimePart *mime_part)
 			
 			/* We want to save textual parts as 8bit instead of encoded */
 			type = camel_data_wrapper_get_mime_type_field (wrapper);
-			if (header_content_type_is (type, "text", "*"))
-				camel_mime_part_set_encoding (mime_part, CAMEL_MIME_PART_ENCODING_8BIT);
+			if (camel_content_type_is (type, "text", "*"))
+				camel_mime_part_set_encoding (mime_part, CAMEL_TRANSFER_ENCODING_8BIT);
 		}
 	}
 }
@@ -2007,11 +2002,8 @@ static void save_messages_saved(struct _mail_msg *mm)
 static void save_messages_free(struct _mail_msg *mm)
 {
 	struct _save_messages_msg *m = (struct _save_messages_msg *)mm;
-	int i;
-
-	for (i=0;i<m->uids->len;i++)
-		g_free(m->uids->pdata[i]);
-	g_ptr_array_free(m->uids, TRUE);
+	
+	em_utils_uids_free (m->uids);
 	camel_object_unref(m->folder);
 	g_free(m->path);
 }
@@ -2087,8 +2079,8 @@ save_part_save (struct _mail_msg *mm)
 	
 	data = camel_medium_get_content_object (CAMEL_MEDIUM (m->part));
 	content_type = camel_mime_part_get_content_type (m->part);
-	if (header_content_type_is (content_type, "text", "*")
-	    && (charset = header_content_type_param (content_type, "charset"))
+	if (camel_content_type_is (content_type, "text", "*")
+	    && (charset = camel_content_type_param (content_type, "charset"))
 	    && strcasecmp (charset, "utf-8") != 0) {
 		charsetfilter = camel_mime_filter_charset_new_convert ("utf-8", charset);
 		filtered_stream = (CamelStream *) camel_stream_filter_new_with_stream (stream_fs);
@@ -2101,7 +2093,7 @@ save_part_save (struct _mail_msg *mm)
 		filtered_stream = stream_fs;
 	}
 	
-	if (camel_data_wrapper_write_to_stream (data, filtered_stream) == -1
+	if (camel_data_wrapper_decode_to_stream (data, filtered_stream) == -1
 	    || camel_stream_flush (filtered_stream) == -1)
 		camel_exception_setv (&mm->ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not write data: %s"),
@@ -2340,3 +2332,101 @@ mail_execute_shell_command (CamelFilterDriver *driver, int argc, char **argv, vo
 	
 	gnome_execute_async_fds (NULL, argc, argv, TRUE);
 }
+
+/* [Un]mark junk flag */
+
+struct _mark_junk_mail_msg {
+	struct _mail_msg msg;
+	
+	CamelFolder *folder;
+	MessageList *list;
+	gboolean junk;
+};
+
+static char *
+mark_junk_describe (struct _mail_msg *mm, int complete)
+{
+	return g_strdup (_("Changing junk status"));
+}
+
+/* filter a folder, or a subset thereof, uses source_folder/source_uids */
+/* this is shared with fetch_mail */
+static void
+mark_junk_mark (struct _mail_msg *mm)
+{
+	struct _mark_junk_mail_msg *m = (struct _mark_junk_mail_msg *) mm;
+	CamelJunkPlugin *csp = NULL;
+	GPtrArray *uids;
+	gboolean commit_reports = FALSE;
+	int i;
+
+	if (m->folder == NULL)
+		return;
+	
+	uids = message_list_get_selected (m->list);
+	camel_folder_freeze (m->folder);
+
+	for (i=0; i<uids->len; i++) {
+		guint32 flags;
+
+		flags = camel_folder_get_message_flags (m->folder, uids->pdata[i]);
+		if (((flags & CAMEL_MESSAGE_JUNK) == CAMEL_MESSAGE_JUNK) != m->junk) {
+			CamelMimeMessage *msg = camel_folder_get_message (m->folder, uids->pdata[i], NULL);
+
+			if (msg) {
+				csp = CAMEL_SERVICE (m->folder->parent_store)->session->junk_plugin;
+				if (m->junk)
+					camel_junk_plugin_report_junk (csp, msg);
+				else
+					camel_junk_plugin_report_notjunk (csp, msg);
+
+				commit_reports = TRUE;
+				camel_object_unref (msg);
+			}
+		}
+		camel_folder_set_message_flags(m->folder, uids->pdata[i],
+					       CAMEL_MESSAGE_JUNK | (m->junk ? CAMEL_MESSAGE_DELETED : 0),
+					       m->junk ? CAMEL_MESSAGE_JUNK : 0);
+	}
+
+	if (commit_reports)
+		camel_junk_plugin_commit_reports (csp);
+
+	message_list_free_uids(m->list, uids);
+	camel_folder_thaw(m->folder);
+}
+
+static void
+mark_junk_marked (struct _mail_msg *mm)
+{
+}
+
+static void
+mark_junk_free (struct _mail_msg *mm)
+{
+	struct _mark_junk_mail_msg *m = (struct _mark_junk_mail_msg *)mm;
+	
+	if (m->folder)
+		camel_object_unref (m->folder);
+}
+
+static struct _mail_msg_op mark_junk_op = {
+	mark_junk_describe,
+	mark_junk_mark,
+	mark_junk_marked,
+	mark_junk_free,
+};
+
+void
+mail_mark_junk (CamelFolder *folder, MessageList *list, gboolean junk)
+{
+	struct _mark_junk_mail_msg *m;
+	
+	m = mail_msg_new (&mark_junk_op, NULL, sizeof (*m));
+	m->folder = folder;
+	camel_object_ref (folder);
+	m->list = list;
+	m->junk = junk;
+	
+	e_thread_put (mail_thread_new, (EMsg *) m);
+}
