@@ -42,10 +42,9 @@
 #include "camel/camel-folder.h"
 #include "camel/camel-operation.h"
 
-#include "evolution-storage.h"
-
 #include "mail.h"
 #include "mail-mt.h"
+#include "mail-component.h"
 #include "mail-config.h"
 #include "mail-session.h"
 #include "mail-tools.h"
@@ -122,7 +121,7 @@ struct _send_info {
 };
 
 static struct _send_data *send_data = NULL;
-static GtkWidget *send_recv_dialogue = NULL;
+static GtkWidget *send_recv_dialog = NULL;
 
 static struct _send_data *setup_send_data(void)
 {
@@ -163,11 +162,10 @@ free_folder_info(void *key, struct _folder_info *info, void *data)
 
 static void free_send_info(void *key, struct _send_info *info, void *data)
 {
-	d(printf("Freeing send info %p\n", info));
 	g_free(info->uri);
 	camel_operation_unref(info->cancel);
 	if (info->timeout_id != 0)
-		gtk_timeout_remove(info->timeout_id);
+		g_source_remove(info->timeout_id);
 	g_free(info->what);
 	g_free(info);
 }
@@ -207,7 +205,7 @@ static void hide_send_info(void *key, struct _send_info *info, void *data)
 	info->status = NULL;
 
 	if (info->timeout_id != 0) {
-		gtk_timeout_remove (info->timeout_id);
+		g_source_remove (info->timeout_id);
 		info->timeout_id = 0;
 	}
 }
@@ -217,11 +215,11 @@ dialog_destroy_cb (struct _send_data *data, GObject *deadbeef)
 {
 	g_hash_table_foreach (data->active, (GHFunc) hide_send_info, NULL);
 	data->gd = NULL;
-	send_recv_dialogue = NULL;
+	send_recv_dialog = NULL;
 }
 
 static void
-dialogue_response(GtkDialog *gd, int button, struct _send_data *data)
+dialog_response(GtkDialog *gd, int button, struct _send_data *data)
 {
 	switch(button) {
 	case GTK_RESPONSE_CANCEL:
@@ -233,7 +231,7 @@ dialogue_response(GtkDialog *gd, int button, struct _send_data *data)
 		gtk_dialog_set_response_sensitive(gd, GTK_RESPONSE_CANCEL, FALSE);
 		break;
 	default:
-		d(printf("hiding dialogue\n"));
+		d(printf("hiding dialog\n"));
 		g_hash_table_foreach(data->active, (GHFunc)hide_send_info, NULL);
 		data->gd = NULL;
 		/*gtk_widget_destroy((GtkWidget *)gd);*/
@@ -287,7 +285,7 @@ static send_info_t get_receive_type(const char *url)
 }
 
 static struct _send_data *
-build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destination)
+build_dialog (EAccountList *accounts, CamelFolder *outbox, const char *destination)
 {
 	GtkDialog *gd;
 	GtkTable *table;
@@ -305,11 +303,12 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 	EAccount *account;
 	EIterator *iter;
 	
-	gd = (GtkDialog *)send_recv_dialogue = gtk_dialog_new_with_buttons(_("Send & Receive Mail"), NULL, 0, NULL);
+	gd = (GtkDialog *)send_recv_dialog = gtk_dialog_new_with_buttons(_("Send & Receive Mail"), NULL, GTK_DIALOG_NO_SEPARATOR, NULL);
+	gtk_window_set_modal ((GtkWindow *) gd, FALSE);
+	
 	stop = (GtkButton *)e_gtk_button_new_with_icon(_("Cancel _All"), GTK_STOCK_CANCEL);
 	gtk_widget_show((GtkWidget *)stop);
 	gtk_dialog_add_action_widget(gd, (GtkWidget *)stop, GTK_RESPONSE_CANCEL);
-	g_object_set(gd, "resizable", FALSE, NULL);
 	gnome_window_icon_set_from_file (GTK_WINDOW (gd), EVOLUTION_ICONSDIR "/send-receive.xpm");
 	
 	num_sources = 0;
@@ -327,11 +326,11 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 	g_object_unref (iter);
 	
 	table = (GtkTable *) gtk_table_new (num_sources, 4, FALSE);
-	gtk_container_set_border_width ((GtkContainer *) table, 6);
+	gtk_container_set_border_width ((GtkContainer *) table, 3);
 	
        	gtk_box_pack_start (GTK_BOX (gd->vbox), GTK_WIDGET (table), TRUE, TRUE, 0);
 	
-	/* must bet setup after send_recv_dialogue as it may re-trigger send-recv button */
+	/* must bet setup after send_recv_dialog as it may re-trigger send-recv button */
 	data = setup_send_data ();
 	
 	row = 0;
@@ -367,7 +366,7 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 			info->keep = source->keep_on_server;
 			info->cancel = camel_operation_new (operation_status, info);
 			info->state = SEND_ACTIVE;
-			info->timeout_id = gtk_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
+			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 			
 			g_hash_table_insert (data->active, info->uri, info);
 			list = g_list_prepend (list, info);
@@ -376,7 +375,7 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 			e_iterator_next (iter);
 			continue;
 		} else if (info->timeout_id == 0)
-			info->timeout_id = gtk_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
+			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 		
 		recv_icon = gtk_image_new_from_file (EVOLUTION_BUTTONSDIR "/receive-24.png");
 		
@@ -413,11 +412,6 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 	
 	g_object_unref (iter);
 	
-	line = (GtkHSeparator *)gtk_hseparator_new ();
-	gtk_table_attach (table, GTK_WIDGET (line), 0, 4, row, row+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 1, 3);
-	row++;
-	gtk_widget_show_all (GTK_WIDGET (table));
-	
 	if (outbox && destination) {
 		info = g_hash_table_lookup (data->active, SEND_URI_KEY);
 		if (info == NULL) {
@@ -429,12 +423,12 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 			info->keep = FALSE;
 			info->cancel = camel_operation_new (operation_status, info);
 			info->state = SEND_ACTIVE;
-			info->timeout_id = gtk_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
+			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 			
 			g_hash_table_insert (data->active, SEND_URI_KEY, info);
 			list = g_list_prepend (list, info);
 		} else if (info->timeout_id == 0)
-			info->timeout_id = gtk_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
+			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 		
 		send_icon  = gtk_image_new_from_file (EVOLUTION_BUTTONSDIR "/send-24.png");
 		
@@ -467,7 +461,7 @@ build_dialogue (EAccountList *accounts, CamelFolder *outbox, const char *destina
 	
 	gtk_widget_show (GTK_WIDGET (gd));
 	
-	g_signal_connect (gd, "response", G_CALLBACK (dialogue_response), data);
+	g_signal_connect (gd, "response", G_CALLBACK (dialog_response), data);
 	
 	g_object_weak_ref ((GObject *) gd, (GWeakNotify) dialog_destroy_cb, data);
 	
@@ -669,19 +663,7 @@ receive_update_got_store (char *uri, CamelStore *store, void *data)
 	struct _send_info *info = data;
 	
 	if (store) {
-		EvolutionStorage *storage = mail_lookup_storage (store);
-		
-		if (storage) {
-			mail_note_store(store, info->cancel, storage, CORBA_OBJECT_NIL, receive_update_done, info);
-			/*bonobo_object_unref (BONOBO_OBJECT (storage));*/
-		} else {
-			/* If we get here, store must be an external
-			 * storage other than /local. (Eg, Exchange).
-			 * Do a get_folder_info just to force it to
-			 * update itself.
-			 */
-			mail_get_folderinfo(store, info->cancel, receive_update_got_folderinfo, info);
-		}
+		mail_get_folderinfo (store, info->cancel, receive_update_got_folderinfo, info);
 	} else {
 		receive_done ("", info);
 	}
@@ -695,24 +677,24 @@ GtkWidget *mail_send_receive (void)
 	EAccount *account;
 	GList *scan;
 	
-	if (send_recv_dialogue != NULL) {
-		if (GTK_WIDGET_REALIZED(send_recv_dialogue)) {
-			gdk_window_show(send_recv_dialogue->window);
-			gdk_window_raise(send_recv_dialogue->window);
+	if (send_recv_dialog != NULL) {
+		if (GTK_WIDGET_REALIZED(send_recv_dialog)) {
+			gdk_window_show(send_recv_dialog->window);
+			gdk_window_raise(send_recv_dialog->window);
 		}
-		return send_recv_dialogue;
+		return send_recv_dialog;
 	}
 	
 	if (!camel_session_is_online (session))
-		return send_recv_dialogue;
+		return send_recv_dialog;
 	
 	account = mail_config_get_default_account ();
 	if (!account || !account->transport->url)
-		return send_recv_dialogue;
+		return send_recv_dialog;
 	
 	accounts = mail_config_get_accounts ();
 	
-	data = build_dialogue (accounts, outbox_folder, account->transport->url);
+	data = build_dialog (accounts, outbox_folder, account->transport->url);
 	scan = data->infos;
 	while (scan) {
 		struct _send_info *info = scan->data;
@@ -744,7 +726,7 @@ GtkWidget *mail_send_receive (void)
 		scan = scan->next;
 	}
 
-	return send_recv_dialogue;
+	return send_recv_dialog;
 }
 
 struct _auto_data {
@@ -776,7 +758,7 @@ static void auto_clean_set(void *key, struct _auto_data *info, GHashTable *set)
 {
 	d(printf("removing auto-check for %s %p\n", info->uri, info));
 	g_hash_table_remove(set, info->uri);
-	gtk_timeout_remove(info->timeout_id);
+	g_source_remove(info->timeout_id);
 	g_free(info->uri);
 	g_free(info);
 }
@@ -817,17 +799,17 @@ mail_autoreceive_setup (void)
 				info->keep = source->keep_on_server;
 				if (info->period != source->auto_check_time*60) {
 					info->period = source->auto_check_time*60;
-					gtk_timeout_remove(info->timeout_id);
-					info->timeout_id = gtk_timeout_add(info->period*1000, auto_timeout, info);
+					g_source_remove(info->timeout_id);
+					info->timeout_id = g_timeout_add(info->period*1000, auto_timeout, info);
 				}
 			} else {
 				info = g_malloc0(sizeof(*info));
 				info->uri = g_strdup(source->url);
 				info->keep = source->keep_on_server;
 				info->period = source->auto_check_time*60;
-				info->timeout_id = gtk_timeout_add(info->period*1000, auto_timeout, info);
+				info->timeout_id = g_timeout_add(info->period*1000, auto_timeout, info);
 				g_hash_table_insert(auto_active, info->uri, info);
-				/* If we do this at startup, it can cause the logon dialogue to be hidden,
+				/* If we do this at startup, it can cause the logon dialog to be hidden,
 				   so lets not */
 				/*mail_receive_uri(source->url, source->keep_on_server);*/
 			}
